@@ -44,7 +44,14 @@
  * @{
  */
 
-int mod_gm_shm_key;        /**< key for the shared memory segment */
+int mod_gm_shm_key;             /**< key for the shared memory segment */
+
+#define SHM_SHIFT             5 /**< nr of global counter              */
+#define SHM_JOBS_DONE         0 /**< shm id for jobs done counter      */
+#define SHM_WORKER_TOTAL      1 /**< shm id for total worker counter   */
+#define SHM_WORKER_RUNNING    2 /**< shm id for running worker counter */
+#define SHM_STATUS_WORKER_PID 3 /**< shm id for status worker pid      */
+#define SHM_WORKER_LAST_CHECK 4 /**< shm time of last check executed   */
 
 /** Mod-Gearman Worker
  *
@@ -124,7 +131,7 @@ int  adjust_number_of_worker(int min, int max, int cur_workers, int cur_jobs);
 void setup_child_communicator(void);
 
 /**
- * finish and clean all childs and shared memory segments, then exit.
+ * finish and clean all children and shared memory segments, then exit.
  *
  * @param[in] sig - signal which caused the exit
  *
@@ -164,7 +171,7 @@ void reload_config(int sig);
  *
  * @return nothing
  */
-void stop_childs(int mode);
+void stop_children(int mode);
 
 /**
  * main loop to maintain the child population
@@ -174,7 +181,7 @@ void stop_childs(int mode);
 void monitor_loop(void);
 
 /**
- * check and start new worker childs if level is too low
+ * check and start new worker children if level is too low
  *
  * @return nothing
  */
@@ -196,6 +203,16 @@ int get_next_shm_index(void);
  */
 void count_current_worker(int restart);
 
+/**
+ * save kill pid from shm index
+ *
+ * @param[in] pid - pid to kill
+ * @param[in] signal - signal to use
+ *
+ * @return nothing
+ */
+void save_kill(int pid, int sig);
+
 /**
  * @}
  */
