@@ -491,7 +491,6 @@ static int handle_host_check( int event_type, void *data ) {
     struct timeval core_time;
     struct tm next_check;
     char buffer1[GM_BUFFERSIZE];
-    int macro_options = STRIP_ILLEGAL_MACRO_CHARS | ESCAPE_MACRO_CHARS;
 
     gettimeofday(&core_time,NULL);
 
@@ -555,14 +554,14 @@ static int handle_host_check( int event_type, void *data ) {
     grab_host_macros(hst);
 
     /* get the raw command line */
-    get_raw_command_line(hst->check_command_ptr,hst->host_check_command,&raw_command,macro_options);
+    get_raw_command_line(hst->check_command_ptr,hst->host_check_command,&raw_command,0);
     if(raw_command==NULL){
         gm_log( GM_LOG_ERROR, "Raw check command for host '%s' was NULL - aborting.\n",hst->name );
         return NEBERROR_CALLBACKCANCEL;
     }
 
     /* process any macros contained in the argument */
-    process_macros(raw_command,&processed_command,macro_options);
+    process_macros(raw_command,&processed_command,0);
     if(processed_command==NULL){
         gm_log( GM_LOG_ERROR, "Processed check command for host '%s' was NULL - aborting.\n",hst->name);
         return NEBERROR_CALLBACKCANCEL;
@@ -623,9 +622,9 @@ static int handle_host_check( int event_type, void *data ) {
     my_free(raw_command);
     my_free(processed_command);
 
-    /* orphanded check - submit fake result to mark host as orphaned */
+    /* orphaned check - submit fake result to mark host as orphaned */
     if(mod_gm_opt->orphan_host_checks == GM_ENABLED && check_options & CHECK_OPTION_ORPHAN_CHECK) {
-        gm_log( GM_LOG_DEBUG, "host check for %s orphanded\n", hst->name );
+        gm_log( GM_LOG_DEBUG, "host check for %s orphaned\n", hst->name );
         if ( ( chk_result = ( check_result * )malloc( sizeof *chk_result ) ) == 0 )
             return NEBERROR_CALLBACKCANCEL;
         snprintf( temp_buffer,GM_BUFFERSIZE-1,"(host check orphaned, is the mod-gearman worker on queue '%s' running?)\n", target_queue);
@@ -665,7 +664,6 @@ static int handle_svc_check( int event_type, void *data ) {
     struct timeval core_time;
     struct tm next_check;
     char buffer1[GM_BUFFERSIZE];
-    int macro_options = STRIP_ILLEGAL_MACRO_CHARS | ESCAPE_MACRO_CHARS;
 
     gettimeofday(&core_time,NULL);
 
@@ -725,14 +723,14 @@ static int handle_svc_check( int event_type, void *data ) {
     grab_service_macros(svc);
 
     /* get the raw command line */
-    get_raw_command_line(svc->check_command_ptr,svc->service_check_command,&raw_command,macro_options);
+    get_raw_command_line(svc->check_command_ptr,svc->service_check_command,&raw_command,0);
     if(raw_command==NULL){
         gm_log( GM_LOG_ERROR, "Raw check command for service '%s' on host '%s' was NULL - aborting.\n", svc->description, svc->host_name );
         return NEBERROR_CALLBACKCANCEL;
     }
 
     /* process any macros contained in the argument */
-    process_macros(raw_command, &processed_command, macro_options);
+    process_macros(raw_command, &processed_command, 0);
     if(processed_command==NULL) {
         gm_log( GM_LOG_ERROR, "Processed check command for service '%s' on host '%s' was NULL - aborting.\n", svc->description, svc->host_name);
         my_free(raw_command);
@@ -803,9 +801,9 @@ static int handle_svc_check( int event_type, void *data ) {
     my_free(raw_command);
     my_free(processed_command);
 
-    /* orphanded check - submit fake result to mark service as orphaned */
+    /* orphaned check - submit fake result to mark service as orphaned */
     if(mod_gm_opt->orphan_service_checks == GM_ENABLED && svc->check_options & CHECK_OPTION_ORPHAN_CHECK) {
-        gm_log( GM_LOG_DEBUG, "service check for %s - %s orphanded\n", svc->host_name, svc->description );
+        gm_log( GM_LOG_DEBUG, "service check for %s - %s orphaned\n", svc->host_name, svc->description );
         if ( ( chk_result = ( check_result * )malloc( sizeof *chk_result ) ) == 0 )
             return NEBERROR_CALLBACKCANCEL;
         snprintf( temp_buffer,GM_BUFFERSIZE-1,"(service check orphaned, is the mod-gearman worker on queue '%s' running?)\n", target_queue);
@@ -840,8 +838,17 @@ static int read_arguments( const char *args_orig ) {
     int verify;
     int errors = 0;
     char *ptr;
-    char *args   = strdup(args_orig);
-    char *args_c = args;
+    char *args;
+    char *args_c;
+
+    if (args_orig == NULL) {
+        gm_log( GM_LOG_ERROR, "error parsing arguments: none provided.\n" );
+        return GM_ERROR;
+    }
+
+    args = strdup(args_orig);
+    args_c = args;
+
     while ( (ptr = strsep( &args, " " )) != NULL ) {
         if(parse_args_line(mod_gm_opt, ptr, 0) != GM_OK) {
             errors++;
