@@ -25,7 +25,7 @@
 /* include header */
 #include "send_multi.h"
 #include "utils.h"
-#include "gearman.h"
+#include "gearman_utils.h"
 
 gearman_client_st client;
 gearman_client_st client_dup;
@@ -139,8 +139,7 @@ int verify_options(mod_gm_opt_t *opt) {
 
     /* no server specified? then default to localhost */
     if(opt->server_num == 0) {
-        opt->server_list[opt->server_num] = strdup("localhost");
-        opt->server_num++;
+        add_server(&opt->server_num, opt->server_list, "localhost");
     }
 
     /* host is mandatory */
@@ -183,7 +182,7 @@ void print_usage() {
     printf("\n");
     printf("             [ --host=<hostname>        ]\n");
     printf("             [ --result_queue=<queue>   ]\n");
-    printf("                 default:result_queue    \n");
+    printf("                 default:check_results   \n");
     printf("             [ --timeout=<seconds>      ]\n");
     printf("\n");
     printf("For details see\n");
