@@ -40,7 +40,6 @@ extern char *p1_file;
 char **start_env;
 #endif
 
-
 /* work starts here */
 #ifdef EMBEDDEDPERL
 int main (int argc, char **argv, char **env) {
@@ -167,6 +166,7 @@ int main (int argc, char **argv) {
     /* maintain worker population */
     monitor_loop();
 
+    gm_log( GM_LOG_ERROR, "worker exited from main loop\n");
     clean_exit(15);
     exit( EXIT_SUCCESS );
 }
@@ -174,17 +174,13 @@ int main (int argc, char **argv) {
 
 /* main loop for checking worker */
 void monitor_loop() {
-    int status;
 
     /* maintain the population */
     while (1) {
         /* check number of workers every second */
         sleep(GM_DEFAULT_WORKER_LOOP_SLEEP);
 
-        /* collect finished workers */
-        while(waitpid(-1, &status, WNOHANG) > 0)
-            gm_log( GM_LOG_TRACE, "waitpid() worker exited with: %d\n", status);
-
+        /* make sure our worker are running */
         check_worker_population();
     }
     return;
@@ -196,7 +192,7 @@ void count_current_worker(int restart) {
     int x;
 
     gm_log( GM_LOG_TRACE3, "count_current_worker()\n");
-    gm_log( GM_LOG_TRACE3, "done jobs:     shm[0] = %d\n", shm[0]);
+    gm_log( GM_LOG_TRACE3, "done jobs:     shm[SHM_JOBS_DONE] = %d\n", shm[SHM_JOBS_DONE]);
 
     /* shm states:
      *   0 -> undefined
@@ -206,16 +202,16 @@ void count_current_worker(int restart) {
      */
 
     /* check if status worker died */
-    if( shm[3] != -1 && pid_alive(shm[3]) == FALSE ) {
-        gm_log( GM_LOG_TRACE, "removed stale status worker, old pid: %d\n", shm[3] );
-        shm[3] = -1;
+    if( shm[SHM_STATUS_WORKER_PID] != -1 && pid_alive(shm[SHM_STATUS_WORKER_PID]) == FALSE ) {
+        gm_log( GM_LOG_TRACE, "removed stale status worker, old pid: %d\n", shm[SHM_STATUS_WORKER_PID] );
+        shm[SHM_STATUS_WORKER_PID] = -1;
     }
-    gm_log( GM_LOG_TRACE3, "status worker: shm[3] = %d\n", shm[3]);
+    gm_log( GM_LOG_TRACE3, "status worker: shm[SHM_STATUS_WORKER_PID] = %d\n", shm[SHM_STATUS_WORKER_PID]);
 
     /* check all known worker */
     current_number_of_workers = 0;
     current_number_of_jobs    = 0;
-    for(x=4; x < mod_gm_opt->max_worker+4; x++) {
+    for(x=SHM_SHIFT; x < mod_gm_opt->max_worker+SHM_SHIFT; x++) {
         /* verify worker is alive */
         gm_log( GM_LOG_TRACE3, "worker slot:   shm[%d] = %d\n", x, shm[x]);
         if( shm[x] != -1 && pid_alive(shm[x]) == FALSE ) {
@@ -235,8 +231,8 @@ void count_current_worker(int restart) {
         }
     }
 
-    shm[1] = current_number_of_workers; /* total worker   */
-    shm[2] = current_number_of_jobs;    /* running worker */
+    shm[SHM_WORKER_TOTAL]   = current_number_of_workers; /* total worker   */
+    shm[SHM_WORKER_RUNNING] = current_number_of_jobs;    /* running worker */
 
     gm_log( GM_LOG_TRACE3, "worker: %d  -  running: %d\n", current_number_of_workers, current_number_of_jobs);
 
@@ -245,15 +241,35 @@ void count_current_worker(int restart) {
 
 /* start new worker if needed */
 void check_worker_population() {
-    int x, now, target_number_of_workers;
+    int x, now, status, target_number_of_workers;
 
     gm_log( GM_LOG_TRACE3, "check_worker_population()\n");
 
+    now = (int)time(NULL);
+
+    /* collect finished workers */
+    while(waitpid(-1, &status, WNOHANG) > 0)
+        gm_log( GM_LOG_TRACE, "waitpid() worker exited with: %d\n", status);
+
     /* set current worker number */
     count_current_worker(GM_ENABLED);
 
+    /* check last check time, force restart all worker if there is no result in 2 minutes */
+    if( shm[SHM_WORKER_LAST_CHECK] < (now - 120) ) {
+        gm_log( GM_LOG_INFO, "no checks in 2minutes, restarting all workers\n", shm[SHM_WORKER_LAST_CHECK]);
+        shm[SHM_WORKER_LAST_CHECK] = now;
+        for(x=SHM_SHIFT; x < mod_gm_opt->max_worker+SHM_SHIFT; x++) {
+            save_kill(shm[x], SIGINT);
+        }
+        sleep(3);
+        for(x=SHM_SHIFT; x < mod_gm_opt->max_worker+SHM_SHIFT; x++) {
+            save_kill(shm[x], SIGKILL);
+            shm[x] = -1;
+        }
+    }
+
     /* check if status worker died */
-    if( shm[3] == -1 ) {
+    if( shm[SHM_STATUS_WORKER_PID] == -1 ) {
         make_new_child(GM_WORKER_STATUS);
     }
 
@@ -263,8 +279,7 @@ void check_worker_population() {
         current_number_of_workers++;
     }
 
-    /* check every second */
-    now = (int)time(NULL);
+    /* check every second if we need to increase worker population */
     if(last_time_increased >= now)
         return;
 
@@ -462,49 +477,56 @@ int verify_options(mod_gm_opt_t *opt) {
 
 /* print usage */
 void print_usage() {
-    printf("usage:\n");
+    printf("Usage: worker [OPTION]...\n");
     printf("\n");
-    printf("worker [ --debug=<lvl>                            ]\n");
-    printf("       [ --logmode=<automatic|stdout|syslog|file> ]\n");
-    printf("       [ --logfile=<path>                         ]\n");
-    printf("       [ --debug-result                           ]\n");
-    printf("       [ --help|-h                                ]\n");
-    printf("       [ --daemon|-d                              ]\n");
-    printf("       [ --config=<configfile>                    ]\n");
-    printf("       [ --server=<server>                        ]\n");
-    printf("       [ --dupserver=<server>                     ]\n");
+    printf("Mod-Gearman worker executes host- and servicechecks.\n");
     printf("\n");
-    printf("       [ --hosts               ]\n");
-    printf("       [ --services            ]\n");
-    printf("       [ --eventhandler        ]\n");
-    printf("       [ --hostgroup=<name>    ]\n");
-    printf("       [ --servicegroup=<name> ]\n");
-    printf("       [ --do_hostchecks       ]\n");
+    printf("Basic Settings:\n");
+    printf("       --debug=<lvl>                                \n");
+    printf("       --logmode=<automatic|stdout|syslog|file>     \n");
+    printf("       --logfile=<path>                             \n");
+    printf("       --debug-result                               \n");
+    printf("       --help|-h                                    \n");
+    printf("       --daemon|-d                                  \n");
+    printf("       --config=<configfile>                        \n");
+    printf("       --server=<server>                            \n");
+    printf("       --dupserver=<server>                         \n");
     printf("\n");
-    printf("       [ --min-worker=<nr>     ]\n");
-    printf("       [ --max-worker=<nr>     ]\n");
+    printf("Encryption:\n");
+    printf("       --encryption=<yes|no>                        \n");
+    printf("       --key=<string>                               \n");
+    printf("       --keyfile=<file>                             \n");
     printf("\n");
-    printf("       [ --max-age=<sec>       ]\n");
-    printf("       [ --timeout             ]\n");
+    printf("Job Control:\n");
+    printf("       --hosts                                      \n");
+    printf("       --services                                   \n");
+    printf("       --eventhandler                               \n");
+    printf("       --hostgroup=<name>                           \n");
+    printf("       --servicegroup=<name>                        \n");
+    printf("       --do_hostchecks                              \n");
+    printf("       --max-age=<sec>                              \n");
+    printf("       --timeout                                    \n");
     printf("\n");
-    printf("       [ --encryption=<yes|no> ]\n");
-    printf("       [ --key=<string>        ]\n");
-    printf("       [ --keyfile=<file>      ]\n");
+    printf("Worker Control:\n");
+    printf("       --min-worker=<nr>                            \n");
+    printf("       --max-worker=<nr>                            \n");
+    printf("       --idle-timeout=<nr>                          \n");
+    printf("       --max-jobs=<nr>                              \n");
+    printf("       --spawn-rate=<nr>                            \n");
+    printf("       --fork_on_exec                               \n");
+    printf("       --load_limit1=load1                          \n");
+    printf("       --load_limit5=load5                          \n");
+    printf("       --load_limit15=load15                        \n");
+    printf("       --show_error_output                          \n");
     printf("\n");
-    printf("       [ --min-worker=<nr>     ]\n");
-    printf("       [ --max-worker=<nr>     ]\n");
-    printf("       [ --idle-timeout=<nr>   ]\n");
-    printf("       [ --max-jobs=<nr>       ]\n");
-    printf("       [ --spawn-rate=<nr>     ]\n");
-    printf("       [ --fork_on_exec        ]\n");
-    printf("       [ --show_error_output   ]\n");
+    printf("Embedded Perl:\n");
+    printf("       --enable_embedded_perl                      \n");
+    printf("       --use_embedded_perl_implicitly              \n");
+    printf("       --use_perl_cache                            \n");
+    printf("       --p1_file                                   \n");
     printf("\n");
-    printf("       [ --enable_embedded_perl         ]\n");
-    printf("       [ --use_embedded_perl_implicitly ]\n");
-    printf("       [ --use_perl_cache               ]\n");
-    printf("       [ --p1_file                      ]\n");
-    printf("\n");
-    printf("       [ --workaround_rc_25    ]\n");
+    printf("Miscellaneous:\n");
+    printf("       --workaround_rc_25\n");
     printf("\n");
     printf("see README for a detailed explaination of all options.\n");
     printf("\n");
@@ -531,12 +553,13 @@ void setup_child_communicator() {
         perror("shmat");
         exit( EXIT_FAILURE );
     }
-    shm[0] = 0;  /* done jobs         */
-    shm[1] = 0;  /* total worker      */
-    shm[2] = 0;  /* running worker    */
-    shm[3] = -1; /* status worker pid */
+    /* change SHM_SHIFT if more global counters are added */
+    shm[SHM_JOBS_DONE]         = 0;  /* done jobs         */
+    shm[SHM_WORKER_TOTAL]      = 0;  /* total worker      */
+    shm[SHM_WORKER_RUNNING]    = 0;  /* running worker    */
+    shm[SHM_STATUS_WORKER_PID] = -1; /* status worker pid */
     for(x = 0; x < mod_gm_opt->max_worker; x++) {
-        shm[x+4] = -1; /* normal worker   */
+        shm[x+SHM_SHIFT] = -1; /* normal worker   */
     }
 
     return;
@@ -548,6 +571,7 @@ int adjust_number_of_worker(int min, int max, int cur_workers, int cur_jobs) {
     int perc_running;
     int idle;
     int target = min;
+    double load[3];
 
     if(cur_workers == 0) {
         gm_log( GM_LOG_TRACE3, "adjust_number_of_worker(min %d, max %d, worker %d, jobs %d) -> %d\n", min, max, cur_workers, cur_jobs, mod_gm_opt->min_worker);
@@ -564,7 +588,24 @@ int adjust_number_of_worker(int min, int max, int cur_workers, int cur_jobs) {
 
     /* > 90% workers running */
     if(cur_jobs > 0 && ( perc_running > 90 || idle <= 2 )) {
-        /* increase target number by 2 */
+        if (getloadavg(load, 3) == -1) {
+            gm_log( GM_LOG_ERROR, "failed to get current load\n");
+            perror("getloadavg");
+        }
+        if(mod_gm_opt->load_limit1 > 0 && load[0] >= mod_gm_opt->load_limit1) {
+            gm_log( GM_LOG_TRACE, "load limit 1min hit, not starting any more workers: %1.2f > %1.2f\n", load[0], mod_gm_opt->load_limit1);
+            return cur_workers;
+        }
+        if(mod_gm_opt->load_limit5 > 0 && load[1] >= mod_gm_opt->load_limit5) {
+            gm_log( GM_LOG_TRACE, "load limit 5min hit, not starting any more workers: %1.2f > %1.2f\n", load[1], mod_gm_opt->load_limit5);
+            return cur_workers;
+        }
+        if(mod_gm_opt->load_limit15 > 0 && load[2] >= mod_gm_opt->load_limit15) {
+            gm_log( GM_LOG_TRACE, "load limit 15min hit, not starting any more workers: %1.2f > %1.2f\n", load[2], mod_gm_opt->load_limit15);
+            return cur_workers;
+        }
+
+        /* increase target number by spawn rate */
         gm_log( GM_LOG_TRACE, "starting %d new workers\n", mod_gm_opt->spawn_rate);
         target = cur_workers + mod_gm_opt->spawn_rate;
     }
@@ -613,7 +654,7 @@ void clean_exit(int sig) {
 void stop_children(int mode) {
     int status, chld;
     int waited = 0;
-    int x, curpid;
+    int x;
 
     gm_log( GM_LOG_TRACE, "stop_children(%d)\n", mode);
 
@@ -627,17 +668,19 @@ void stop_children(int mode) {
      */
     killpg(0, SIGTERM);
     while(current_number_of_workers > 0) {
+
         gm_log( GM_LOG_TRACE, "send SIGTERM\n");
-        for(x=3; x < mod_gm_opt->max_worker+4; x++) {
-            curpid = shm[x];
-            if(curpid < 0) { curpid = -curpid; }
-            if( curpid != 0 && curpid != 1 ) {
-                kill(curpid, SIGTERM);
-            }
+        save_kill(shm[SHM_STATUS_WORKER_PID], SIGTERM);
+        for(x=SHM_SHIFT; x < mod_gm_opt->max_worker+SHM_SHIFT; x++) {
+            save_kill(shm[x], SIGTERM);
         }
         while((chld = waitpid(-1, &status, WNOHANG)) != -1 && chld > 0) {
             gm_log( GM_LOG_TRACE, "wait() %d exited with %d\n", chld, status);
         }
+
+        if(mode == GM_WORKER_RESTART)
+            break;
+
         sleep(1);
         waited++;
         if(waited > GM_CHILD_SHUTDOWN_TIMEOUT) {
@@ -645,7 +688,7 @@ void stop_children(int mode) {
         }
         count_current_worker(GM_DISABLED);
         if(current_number_of_workers == 0)
-            return;
+            break;
         gm_log( GM_LOG_TRACE, "still waiting (%d) %d children missing...\n", waited, current_number_of_workers);
     }
 
@@ -656,12 +699,9 @@ void stop_children(int mode) {
             return;
 
         gm_log( GM_LOG_TRACE, "sending SIGINT...\n");
-        for(x=3; x < mod_gm_opt->max_worker+4; x++) {
-            curpid = shm[x];
-            if(curpid < 0) { curpid = -curpid; }
-            if( curpid != 0 && curpid != 1 ) {
-                kill(curpid, SIGINT);
-            }
+        save_kill(shm[SHM_STATUS_WORKER_PID], SIGINT);
+        for(x=SHM_SHIFT; x < mod_gm_opt->max_worker+SHM_SHIFT; x++) {
+            save_kill(shm[x], SIGINT);
         }
 
         /* wait 3 more seconds*/
@@ -676,14 +716,9 @@ void stop_children(int mode) {
         count_current_worker(GM_DISABLED);
         if(current_number_of_workers == 0)
             return;
-        for(x=3; x < mod_gm_opt->max_worker+4; x++) {
-            if( shm[x] != 0 ) {
-                curpid = shm[x];
-                if(curpid < 0) { curpid = -curpid; }
-                if( curpid != 0 ) {
-                    kill(curpid, SIGKILL);
-                }
-            }
+        save_kill(shm[SHM_STATUS_WORKER_PID], SIGKILL);
+        for(x=SHM_SHIFT; x < mod_gm_opt->max_worker+SHM_SHIFT; x++) {
+            save_kill(shm[x], SIGKILL);
         }
 
         /* count children a last time */
@@ -778,6 +813,9 @@ void reload_config(int sig) {
     /* start status worker */
     make_new_child(GM_WORKER_STATUS);
 
+    /* start normal worker */
+    check_worker_population();
+
     gm_log( GM_LOG_INFO, "reloading config was successful\n");
 
     return;
@@ -791,7 +829,7 @@ int get_next_shm_index() {
 
     gm_log( GM_LOG_TRACE, "get_next_shm_index()\n" );
 
-    for(x = 4; x < mod_gm_opt->max_worker+4; x++) {
+    for(x = SHM_SHIFT; x < mod_gm_opt->max_worker+SHM_SHIFT; x++) {
         if(shm[x] == -1) {
             next_index      = x;
             shm[next_index] = 1;
@@ -809,6 +847,15 @@ int get_next_shm_index() {
     return next_index;
 }
 
+/* kill child */
+void save_kill(int pid, int sig) {
+    if(pid < 0) { pid = -pid; }
+    if( pid != 0 && pid != 1 ) {
+        kill(pid, sig);
+    }
+    return;
+}
+
 
 /* core log wrapper */
 void write_core_log(char *data) {
