@@ -83,9 +83,11 @@ void worker_client(int worker_mode, int indx, int shid) {
     }
 
     /* create duplicate client */
-    if ( create_client_dup( mod_gm_opt->dupserver_list, &client_dup ) != GM_OK ) {
-        gm_log( GM_LOG_ERROR, "cannot start client for duplicate server\n" );
-        _exit( EXIT_FAILURE );
+    if( mod_gm_opt->dupserver_num ) {
+        if ( create_client_dup( mod_gm_opt->dupserver_list, &client_dup ) != GM_OK ) {
+            gm_log( GM_LOG_ERROR, "cannot start client for duplicate server\n" );
+            _exit( EXIT_FAILURE );
+        }
     }
 
 #ifdef EMBEDDEDPERL
@@ -454,6 +456,8 @@ void set_state(int status) {
     if(status == GM_JOB_END) {
         shm[SHM_JOBS_DONE]++; /* increase jobs done */
 
+        shm[SHM_WORKER_LAST_CHECK] = (int)time(NULL); /* set last job date */
+
         /* status slot changed to -1 -> exit */
         if( shm[shm_index] == -1 ) {
             gm_log( GM_LOG_TRACE, "worker finished: %d\n", getpid() );
